MY_STOP_WORDS = {
    "en": set(
        """
like yeah know thing oh ok
""".split()
    ),
    "de": set(
        """
oh ok
mal einfach sagen eigentlich finden genau
quasi halt hey irgendwie bisschen bissch
beispielsweise irgendjemand all worum
""".split()
    )
}
